package ru.dbotthepony.mc.otm.block.entity

import net.minecraft.world.level.block.entity.BlockEntityType
import net.minecraft.core.BlockPos
import net.minecraft.world.level.block.state.BlockState
import net.minecraft.world.MenuProvider
import java.lang.Runnable
import net.minecraft.server.level.ServerLevel
import net.minecraft.client.multiplayer.ClientLevel
import net.minecraft.core.SectionPos
import net.minecraft.world.entity.player.Inventory
import net.minecraft.world.entity.player.Player
import net.minecraft.world.inventory.AbstractContainerMenu
import net.minecraft.nbt.CompoundTag
import net.minecraft.network.chat.Component
import net.minecraft.world.level.Level
import net.minecraft.world.level.block.entity.RandomizableContainerBlockEntity
import ru.dbotthepony.mc.otm.core.TextComponent
import ru.dbotthepony.mc.otm.core.nbt.getJson
import ru.dbotthepony.mc.otm.core.nbt.putJson
import ru.dbotthepony.mc.otm.core.nbt.set
import ru.dbotthepony.mc.otm.oncePre

/**
 * Device block entity base, implementing [MenuProvider] and [IRedstoneControlled], and also tracks custom display name
 */
abstract class MatteryDeviceBlockEntity(blockEntityType: BlockEntityType<*>, blockPos: BlockPos, blockState: BlockState)
	: MatteryBlockEntity(blockEntityType, blockPos, blockState), MenuProvider, IRedstoneControlled {
	var customDisplayName: Component? = null

	override val redstoneControl: AbstractRedstoneControl = RedstoneControl { new, old ->
		setChangedLight()

		if (new != old)
			redstoneStatusUpdated(new, old)
	}

	protected open val defaultDisplayName: Component
		get() = level?.getBlockState(blockPos)?.block?.name ?: TextComponent("null at $blockPos")

	abstract override fun createMenu(containerID: Int, inventory: Inventory, ply: Player): AbstractContainerMenu?

	override fun getDisplayName(): Component {
		return customDisplayName ?: defaultDisplayName
	}

	protected open fun redstoneStatusUpdated(newBlocked: Boolean, oldBlocked: Boolean) {}

	override fun saveAdditional(nbt: CompoundTag) {
		super.saveAdditional(nbt)

		if (customDisplayName != null)
			nbt.putJson("Name", Component.Serializer.toJsonTree(customDisplayName!!))

		nbt[REDSTONE_CONTROL_KEY] = redstoneControl.serializeNBT()
	}

	override fun load(nbt: CompoundTag) {
		super.load(nbt)

		customDisplayName = nbt.getJson("Name")?.let(Component.Serializer::fromJson)
		redstoneControl.deserializeNBT(nbt[REDSTONE_CONTROL_KEY] as? CompoundTag)
	}
}
